import Decimal from 'decimal.js';
import { Hash, Address, LocalAccount } from 'viem';
import { SmartAccount } from 'viem/account-abstraction';

type Account = LocalAccount | SmartAccount;
declare const TESTNET_RPC_URL = "https://testnet-rpc.foundation.network/perpetual";
type ClientOptions = {
    rpcUrl: string;
    subAccountIndex: number;
    brokerId: number;
};
declare class FoundationPerpClient {
    private rpcClient;
    private markets;
    private signer;
    subaccount: Hash;
    constructor(signer: Account, options_: Partial<ClientOptions>);
    getAccountInfo(): Promise<AccountInfo>;
    getPendingOrders(symbol: string): Promise<PendingOrder[]>;
    getMarketState(symbol: string): Promise<any>;
    getMarket(ticker: string): Promise<MarketConfig>;
    getOrderbookDepth(symbol: string, limit: number): Promise<any>;
    placeLimit(market: string, side: Side, price: string, amount: string, flag_?: Partial<OrderFlag> | undefined): Promise<any>;
    cancelOrder(market: string, orderId: number): Promise<any>;
    private getMarketConfig;
}
type MarketConfig = {
    symbol: Hash;
    ticker: string;
    tickSize: Decimal;
    stepSize: Decimal;
    minAmount: Decimal;
    offchainBook: Address;
};
type Side = 'ask' | 'bid';
declare const TIME_IN_FORCES: readonly ["default", "immediate_or_cancel", "fill_or_kill", "post_only"];
declare const SELF_TRADE_BEHAVIORS: readonly ["cancel_provide", "decrease_take", "expire_both", "fill"];
type TimeInForce = (typeof TIME_IN_FORCES)[number];
type SeltTradeBehavior = (typeof SELF_TRADE_BEHAVIORS)[number];
type OrderFlag = {
    timeInForce: TimeInForce;
    selfTradeBehavior: SeltTradeBehavior;
    reduceOnly: boolean;
    expiresAt: number | undefined | null;
    isMarketOrder: boolean;
};
type NumericString = `${number}`;
type OrderStatus = 'placed' | 'filled' | 'canceled' | 'conditional_canceled';
type PendingOrder = {
    order_id: 8;
    account_id: Hash;
    symbol: Hash;
    side: Side;
    create_timestamp: number;
    amount: NumericString;
    price: NumericString;
    status: OrderStatus;
    matched_quote_amount: NumericString;
    matched_base_amount: NumericString;
    quote_fee: NumericString;
    nonce: number;
    expiration: OrderFlag;
    trigger_condition: null;
    is_triggered: true;
    signature: Hash;
    signer: Address | null;
    hash: Hash;
    has_dependency: boolean;
    tag: 'limit' | 'market' | 'stop_loss' | 'take_profit';
};
type Position = {
    base_amount: NumericString;
    quote_amount: NumericString;
    last_cumulative_funding: NumericString;
    frozen_in_bid_order: NumericString;
    frozen_in_ask_order: NumericString;
    unsettled_pnl: NumericString;
    is_settle_pending: boolean;
};
type AccountInfo = {
    positions: Record<Hash, Position>;
    collateral: NumericString;
    is_in_liquidation_queue: boolean;
};
type Level = [NumericString, NumericString, NumericString];
type Depth = {
    asks: Level[];
    bids: Level[];
    symbol: Hash;
};

export { type AccountInfo, type Depth, FoundationPerpClient, type Level, type MarketConfig, type NumericString, type OrderStatus, type PendingOrder, type Position, type SeltTradeBehavior, TESTNET_RPC_URL, type TimeInForce };
