'use client'
"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  FoundationPerpClient: () => FoundationPerpClient,
  TESTNET_RPC_URL: () => TESTNET_RPC_URL
});
module.exports = __toCommonJS(src_exports);
var import_client_js = __toESM(require("@open-rpc/client-js"));
var import_decimal = __toESM(require("decimal.js"));
var import_node_assert = __toESM(require("assert"));
var import_node_crypto = require("crypto");
var import_viem = require("viem");

// src/utils.ts
var buildAccountId = (address, brokerId, subaccountIndex) => {
  return `${address}${`${brokerId}`.padStart(8, "0")}000000000001${`${subaccountIndex}`.padStart(4, "0")}`;
};

// src/index.ts
var TESTNET_RPC_URL = "https://testnet-rpc.foundation.network/perpetual";
var DefaultClientOption = {
  rpcUrl: TESTNET_RPC_URL,
  brokerId: 1,
  subAccountIndex: 0
};
var EIP712_DOMAIN = {
  name: "FOUNDATION",
  chainId: 1,
  version: "0.1.0"
};
var FoundationPerpClient = class {
  constructor(signer, options_) {
    this.markets = [];
    const options = Object.assign(
      {},
      DefaultClientOption,
      options_
    );
    const transport = options.rpcUrl.startsWith("ws") ? new import_client_js.WebSocketTransport(options.rpcUrl) : new import_client_js.HTTPTransport(options.rpcUrl);
    this.rpcClient = new import_client_js.default(new import_client_js.RequestManager([transport]));
    this.signer = signer;
    this.subaccount = buildAccountId(
      signer.address,
      options.brokerId,
      options.subAccountIndex
    );
  }
  async getAccountInfo() {
    return await this.rpcClient.request({
      method: "ob_query_account",
      params: [this.subaccount]
    });
  }
  async getPendingOrders(symbol) {
    const marketConfig = await this.getMarket(symbol);
    return await this.rpcClient.request({
      method: "ob_query_user_orders",
      params: [marketConfig.symbol, this.subaccount]
    });
  }
  async getMarketState(symbol) {
    const marketConfig = await this.getMarket(symbol);
    const states = await this.rpcClient.request({
      method: "ob_query_markets_state"
    });
    return states.find((x) => x.symbol === marketConfig.symbol);
  }
  async getMarket(ticker) {
    if (!this.markets.length) {
      await this.getMarketConfig();
    }
    const market = this.markets.find(
      (x) => x.ticker === ticker
    );
    (0, import_node_assert.default)(market, "unknown ticker");
    return market;
  }
  async getOrderbookDepth(symbol, limit) {
    const marketConfig = await this.getMarket(symbol);
    return await this.rpcClient.request({
      method: "ob_query_depth",
      params: [marketConfig.symbol, limit]
    });
  }
  async placeLimit(market, side, price, amount, flag_) {
    const marketConfig = await this.getMarket(market);
    const flag = typeof flag_ === "undefined" ? DEFAULT_ORDER_FLAG : Object.assign({}, DEFAULT_ORDER_FLAG, flag_);
    const cmd = await createPostCmd(
      marketConfig,
      this.signer,
      this.subaccount,
      side,
      new import_decimal.default(price),
      new import_decimal.default(amount),
      flag
    );
    return await this.rpcClient.request(cmd);
  }
  async cancelOrder(market, orderId) {
    const marketConfig = await this.getMarket(market);
    const nonce = BigInt(Date.now() + (0, import_node_crypto.randomInt)(1e3, 3e4)) << 20n;
    const signData = {
      domain: {
        ...EIP712_DOMAIN,
        verifyingContract: marketConfig.offchainBook
      },
      message: {
        subaccount: this.subaccount,
        nonce,
        orderId: BigInt(orderId)
      },
      primaryType: "Cancel",
      types: {
        Cancel: [
          { type: "bytes32", name: "subaccount" },
          { type: "uint64", name: "nonce" },
          { type: "uint64", name: "orderId" }
        ]
      }
    };
    const signature = await this.signer.signTypedData(signData);
    return await this.rpcClient.request({
      method: "ob_trade",
      params: [
        this.subaccount,
        {
          cancel: {
            symbol: marketConfig.symbol,
            orderId: orderId.toString()
          }
        },
        signature,
        nonce.toString(),
        ""
      ]
    });
  }
  async getMarketConfig() {
    const markets = await this.rpcClient.request({
      method: "ob_query_open_markets",
      params: []
    });
    this.markets = markets.map((t) => {
      return {
        symbol: t.symbol,
        ticker: t.ticker,
        offchainBook: t.offchain_book,
        tickSize: new import_decimal.default(t.tick_size),
        stepSize: new import_decimal.default(t.step_size),
        minAmount: new import_decimal.default(t.step_size)
      };
    });
  }
};
var TIME_IN_FORCES = [
  "default",
  "immediate_or_cancel",
  "fill_or_kill",
  "post_only"
];
var SELF_TRADE_BEHAVIORS = [
  // cancel the maker order and continue to fill taker
  "cancel_provide",
  // cancel the remaining of the taker, stop filling
  "decrease_take",
  // abort the fill process, revert all filled and force cancel the maker order
  "expire_both",
  // allow self trade
  "fill"
];
var DEFAULT_ORDER_FLAG = {
  expiresAt: void 0,
  reduceOnly: false,
  timeInForce: "default",
  isMarketOrder: false,
  selfTradeBehavior: "cancel_provide"
};
var encodeFlag = (flag) => {
  const isMarket = flag.isMarketOrder ? 1n : 0n;
  const tif = BigInt(TIME_IN_FORCES.indexOf(flag.timeInForce));
  const stb = BigInt(SELF_TRADE_BEHAVIORS.indexOf(flag.selfTradeBehavior));
  return tif << 62n | isMarket << 60n | stb << 58n | BigInt(flag.expiresAt || 0);
};
var createPostCmd = async (market, signer, subaccount, method, price, amount, flag) => {
  const nonce = BigInt(Date.now() + (0, import_node_crypto.randomInt)(1e3, 3e4)) << 20n;
  const exp = encodeFlag(flag);
  const signData = {
    domain: { ...EIP712_DOMAIN, verifyingContract: market.offchainBook },
    message: {
      subaccount,
      nonce,
      price: (0, import_viem.parseEther)(price.toFixed(18)),
      amount: method === "bid" ? (0, import_viem.parseEther)(amount.toFixed(18)) : -(0, import_viem.parseEther)(amount.toFixed(18)),
      expiration: exp,
      triggerCondition: 0n
    },
    primaryType: "Order",
    types: {
      Order: [
        { name: "subaccount", type: "bytes32" },
        { name: "price", type: "int256" },
        { name: "amount", type: "int256" },
        { name: "nonce", type: "uint64" },
        { name: "expiration", type: "uint64" },
        { name: "triggerCondition", type: "uint128" }
      ]
    }
  };
  const signature = await signer.signTypedData(signData);
  return {
    method: "ob_trade",
    params: [
      subaccount,
      // account id
      {
        [method]: {
          symbol: market.symbol,
          amount,
          price,
          expiration: {
            time_in_force: flag.timeInForce,
            // immediate_or_cancel | fill_or_kill | post_only
            reduce_only: flag.reduceOnly,
            expires_at: flag.expiresAt,
            // unix epoch in second
            is_market_order: flag.isMarketOrder,
            self_trade_behavior: flag.selfTradeBehavior
          }
        }
      },
      signature,
      nonce.toString(),
      ""
    ]
  };
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  FoundationPerpClient,
  TESTNET_RPC_URL
});
